/* -*- mode: c++; c-basic-offset: 4; -*- */
#include "file_driver.hh"
#include "loc_list.hh"
#include <stdexcept>

#include "ldas/ldasconfig.hh"
#ifndef LDAS_VERSION_NUMBER
#define LDAS_VERSION_NUMBER 109000
#endif
#include "framecpp/IFrameStream.hh"

#include <fstream>
#include <iostream>
#include <unistd.h>

using namespace std;
using namespace sends;

//======================================  Constructor
file_driver::file_driver(void) {
}

//======================================  Destructor
file_driver::~file_driver(void) {
}

//======================================  Read data into segments
int
file_driver::read_data(const loc_list& ll, gps_type start, 
		       gps_type stop, const channel_vect& v) {
    int rc = 0;

    const frame_loc* fl = ll.find_loc(start);
    if (!fl) {
	cerr << "GPS not in loc_list" << endl;
	return -1;
    }
    if (stop > fl->stop()) stop = fl->stop();
    string prefix = fl->prefix();
    ll.unlock();

    //----------------------------------  Loop over frames
    segment_vect segv;
    while (start < stop) {
	fl = ll.find_loc(start);
	if (!fl) {
	    cerr << "GPS not in loc_list" << endl;
	    rc = -1;
	    break;
	}

	string file = fl->file_path(start);
	gps_type end_frame   = fl->frame_end(start);
	gps_type start_frame = fl->frame_start(start);
	ll.unlock();

	if (file.empty()) {
	    cerr << "file_driver: Can't find frame for gps= " << start 
		 << " in " << loc_list::arch_name(ll.archive()) << ":" 
		 << prefix << endl;
	    rc = -1;
	    break;
	} else if (access(file.c_str(), R_OK) != 0) {
	    cerr << "file_driver: Can't find frame: " << file << endl;
	    rc = -1;
	    break;
	}

	if (!get_segments(start_frame, end_frame, v, segv)) {
	    start = end_frame;
	    continue;
	}

	try {
	    //--------------------------  Open the frame
	    if (debug() >= 2) cerr << "Read data from frame: " << file << endl;
#if LDAS_VERSION_NUMBER < 109360
	    ifstream frstr(file.c_str());
	    FrameCPP::IFrameStream frdr(frstr);
#else
	    FrameCPP::Common::FrameBuffer<filebuf>* 
		frbuf = new FrameCPP::Common::FrameBuffer<filebuf>(ios::in);
	    if (! frbuf->open(file.c_str(), ios::in) ) 
		throw runtime_error( string("Unable to open file: ") + file);
	    FrameCPP::IFrameStream frdr(frbuf);
#endif

	    //--------------------------  Loop over frames, read data.
	    int nFrames = frdr.GetTOC()->nFrame();
	    for (int iFrame=0; iFrame < nFrames; ++iFrame) {
		if (read_frame(frdr, segv, iFrame, start_frame)) {
		    cerr << "Error reading frame: " << file << endl;
		    rc = -1;
		}
	    }

	    //--------------------------  Close frame file.
#if LDAS_VERSION_NUMBER < 109360
	    frstr.close();
#else
	    frbuf->close();
#endif
	    start = end_frame;

	//------------------------------  On error, release segments and 
	//                                rethrow exception
	} catch (std::exception& e) {
	    release_segments(segv);
	    cerr << "file_driver::read_data: caught exception: " 
		 << e.what() << endl;
	    rc = -1;
	    break;
	} catch (...) {
	    release_segments(segv);
	    throw;
	}
	release_segments(segv);
    }
    return rc;
}
